#ifndef LIST_H
#define LIST_H

#include "Exceptions.h"
#include <iostream>
using namespace std;

template <typename T>
class List {
public:
	List();
	~List();

	void addElement(T element);
	T removeElement(int position);
	void removeAllElements();

	bool isEmpty();
	int getSize();

	void print();

private:

	struct Elem {
		T info;
		Elem* next;
		Elem* prev;
		Elem(T info, Elem* prev, Elem* next) {
			this->info = info;
			this->prev = prev;
			this->next = next;
		}
	};

	Elem* first, * last;
	int size;
	T x;

};

template <typename T>
List<T>::List() {
	this->size = 0;
	this->first = this->last = nullptr;
}

template <typename T>
List<T>::~List() {
	this->removeAllElements();
}

template <typename T>
void List<T>::addElement(T element) {
	if (!this->first) {
		this->last = this->first = new Elem(element, nullptr, nullptr);
	}
	else {
		this->last = this->last->next = new Elem(element, this->last, nullptr);
	}
	this->size++;
}

template <typename T>
bool List<T>::isEmpty() {
	return this->size == 0;
}

template <typename T>
int List<T>::getSize() {
	return this->size;
}

template <typename T>
T List<T>::removeElement(int position) {
	Elem* current = this->first;

	int i = 0;
	for (; current && i < position; i++)
		current = current->next;

	if (!current || i != position) { throw new IndexOutOfBoundsException(); }

	if (current->prev) current->prev->next = current->next;
	if (current->next) current->next->prev = current->prev;
	if (current == this->first) this->first = this->first->next;
	if (current == this->last) this->last = this->last->prev;
	T element = current->info;
	delete current;
	this->size--;
	return element;
}

template <typename T>
void List<T>::removeAllElements() {
	while (this->first) {
		Elem* tmp = this->first;
		this->first = this->first->next;
		delete tmp;
	}
	this->last = nullptr;
	this->size = 0;
}

template <typename T>
void List<T>::print() {
	cout << "List: " << endl;
	Elem* current = this->first;
	while (current) {
		current->info->print();
		current = current->next;
	}
}

#endif
